package ALTLinux::ACL;
use strict;
use warnings;
use Carp;
use File::Path qw(make_path);
use File::Basename;
use IPC::Run3;

our $VERSION = 0.224;
our $PKGNAME2ACL;
our $GRPNAME2ACL;
my $aclpackageprefix='list.packages.';
my $aclgroupprefix='list.groups.';
my @cachedirs=grep {$_} ($ENV{HOME}.'/.cache/altlinux-acl','.');

our $default_expire = 5/24; # -M; 5/24=5 hours
our $default_branch = 'sisyphus';
our $default_verbose = 1;
our $default_offline = $ENV{'ALTLINUX_ACL_OFFLINE'};

our @LONGOPT = (
    'acl-branch=s' => \$default_branch,
    'acl-expire=f' => \$default_expire,
    'acl-offline!' => \$default_offline,
    'acl-verbose=i' => \$default_verbose,
    );

sub print_longopt {
print "    ALTLinux::ACL options:
--acl-branch sisyphus|p10|... Name of branch. Default is sisyphus.
--acl-expire days Cache expiration time, float, in days. Default is 1/5 day.
--acl-offline Assume offline. Use stored acl cache, ignoring expiration time.
--acl-verbose <verbosity level> Internal debug/verbosity.
";
}

sub new {
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self = {
	-expire=> $default_expire, # -M; 5/24=5 hours
	-branch=> $default_branch,
	-offline=> $default_offline,
	-verbose=> $default_verbose,
	@_,
    };
    bless($self,$class);
    unless ($PKGNAME2ACL) {
	$PKGNAME2ACL=$self->__load(-packages,$aclpackageprefix);
	$GRPNAME2ACL=$self->__load(-groups,$aclgroupprefix);
	# save from expansion
	$GRPNAME2ACL->{'@nobody'}=['@nobody'];
	$GRPNAME2ACL->{'@everybody'}=['@everybody'];
    }
    if ($self->{-verbose} and not $self->is_acl_exist()) {
	print STDERR "Note: acl for $self->{-branch} is empty\n";
    }
    return $self;
}

sub is_acl_exist {
    my ($self)=@_;
    return $self->{-packages_non_empty};
}

sub acl {
    my ($self,$item)=@_;
    Carp::carp "ALTLinux::ACL->acl: got undefined pkgname argument!" if not defined $item;
    my $ptr=$PKGNAME2ACL->{$item};
    return unless $ptr;
    return @$ptr;
}

sub group_acl {
    my ($self,$item)=@_;
    Carp::carp "ALTLinux::ACL->group_acl: got undefined group name argument!" if not defined $item;
    my $ptr=$GRPNAME2ACL->{$item};
    return unless $ptr;
    return @$ptr;
}

sub acl_with_groups_expanded {
    my ($self,$item)=@_;
    Carp::carp "ALTLinux::ACL->acl: got undefined pkgname argument!" if not defined $item;
    my $ptr=$PKGNAME2ACL->{$item};
    return unless $ptr;
    return map {$GRPNAME2ACL->{$_} ? @{$GRPNAME2ACL->{$_}} : $_ } @$ptr;
}

sub is_authorized {
    my ($self,$item, $user)=@_;
    Carp::carp "ALTLinux::ACL->is_authorized: got undefined pkgname argument!" if not defined $item;
    Carp::carp "ALTLinux::ACL->is_authorized: got undefined user argument!" if not defined $user;
    my %expanded = map {$_=>1} $self->acl_with_groups_expanded($item);
    return $expanded{$user} || $expanded{'@everybody'} || $expanded{'@nobody'};
}

sub is_leader {
    my ($self,$item, $user)=@_;
    Carp::carp "ALTLinux::ACL->is_leader: got undefined pkgname argument!" if not defined $item;
    Carp::carp "ALTLinux::ACL->is_leader: got undefined user argument!" if not defined $user;
    my $raw_leader=$PKGNAME2ACL->{$item}->[0];
    return 0 unless defined $raw_leader;
    return 1 if $raw_leader eq $user;
    return 0 if substr($raw_leader, 0, 1) ne '@';
    # indirect leader
    $raw_leader=$GRPNAME2ACL->{$raw_leader}->[0];
    return $raw_leader eq $user;
}

sub leader {
    my ($self,$item)=@_;
    Carp::carp "ALTLinux::ACL->leader: got undefined pkgname argument!" if not defined $item;
    my $raw_leader;
    my $acllist=$PKGNAME2ACL->{$item};
    $raw_leader=$acllist->[0] if $acllist;
    unless ($raw_leader) {
	$acllist=$GRPNAME2ACL->{$item};
	$raw_leader=$acllist->[0] if $acllist;
    }
    return $raw_leader;
}

sub pkg_leader {
    my ($self,$item)=@_;
    Carp::carp "ALTLinux::ACL->leader: got undefined pkgname argument!" if not defined $item;
    my $acllist=$PKGNAME2ACL->{$item};
    return $acllist ? $acllist->[0]: undef;
}

sub group_leader {
    my ($self,$item)=@_;
    Carp::carp "ALTLinux::ACL->leader: got undefined group argument!" if not defined $item;
    my $acllist=$GRPNAME2ACL->{$item};
    return $acllist ? $acllist->[0]: undef;
}

sub is_group {
    my ($self,$aclentry)=@_;
    Carp::carp "ALTLinux::ACL->is_group: got undefined acl argument!" if not defined $aclentry;
    return ! defined($aclentry) || (substr($aclentry,0,1) eq '@');
}

sub pkglist($) {
    my ($self)=@_;
    return keys(%$PKGNAME2ACL);
}

sub grplist($) {
    my ($self)=@_;
    return keys(%$GRPNAME2ACL);
}

############ PUBLIC STATIC ###############
# i.e. called ALTLinux::ACL::<sub name>

sub email2aclname {
    my ($email)=@_;
    $email=~/^.+\<(\S+)\s*(?:\@|at)\s*altlinux.*>/;
    my $user=$1;
    unless ($user) {
	warn "e-mail [$email] does not look like ALT Linux maintainer e-mail.";
	return;
    }
    # for php-coder (acl name php_coder).
    $user=~s/-/_/g;
    return $user;
}

########## PRIVATE #################

sub __load {
    my ($self, $aclopt, $aclprefix)=@_;
    my $cache={};
    # let us tolerate 'Sisyphus', etc, to...
    my $branch=lc($self->{'-branch'});
    my $aclfile=$self->{$aclopt};
    if (not defined $aclfile) {
	my @acllist = grep {defined($_) and -e $_} $self->{$aclopt}, map {$_.'/'.$aclprefix.$branch} @cachedirs;
	$aclfile||=$acllist[0];
    }
    $aclfile||=$cachedirs[0].'/'.$aclprefix.$branch;

    # $aclfile contains creation timestamp, useful for wget speedup.
    # this is check timestamp.
    my $timestampfile=$aclfile.'.timestamp';
    if (not -e $aclfile or not -e $timestampfile or (not $self->{-offline} and -M $timestampfile > $self->{-expire})) {
	make_path(dirname($aclfile));
	my $url='http://git.altlinux.org/acl/'.$aclprefix.$branch;
	my $aclfile_new=$aclfile.'.new';
	my $wget_stderr;
	print STDERR "updating acl cache: wget -q -O ",$aclfile_new,' ',$url,"\n" if $self->{-verbose};
	run3 ['wget', '-q', '-O'.$aclfile_new, $url], \undef, \undef, \$wget_stderr;
	my $retval=$?;
	if ($retval==0 and -e $aclfile_new and -s $aclfile_new) {
	    rename ($aclfile_new, $aclfile) || die "can't rename acl file ",$aclfile_new,' to ',$aclfile,"\n";
	} elsif (-e $aclfile and -s $aclfile) {
	    unlink $aclfile_new;
	    warn "WARNING: can't update acl: wget -q -O",$aclfile_new,' ',$url,' failed (',$retval,'): ',$wget_stderr,"\n";
	} else {
	    # not -e or not -s $aclfile
	    unlink $aclfile, $aclfile_new;
	    die "can't download acl file $url to $aclfile\n";
	}
	die "can't find acl file ",$aclfile unless -e $aclfile;
	system ('touch', $timestampfile);
    }

    open (ACL,'<',$aclfile) || die "can't open acl file $aclfile: $!";
    while (<ACL>) {
	chomp;
	if (/([^\t]+)\t(.+)/) {
	    my $name=$1;
	    my @acl=split (/\s+/,$2);
	    $cache->{$name}=\@acl if @acl;
	} else {
	    warn "acl: strange string $_";
	}
    }
    close(ACL);
    $self->{$aclopt}=$aclfile;
    $self->{$aclopt.'_non_empty'}=1 if %$cache;
    return $cache;
}

1;

=head1	NAME

ALTLinux::ACL - ALTLinux ACL query library.

=head1	DESCRIPTION

ALTLinux ACL query library automatically downloads, caches, and
provides an object interface to ALTLinux acl files.

=head1	OPTIONS

=over

=item	B<--acl-branch> I<sisyphus|t7|...>

Name of branch. Default is sisyphus.

=item	B<--acl-expire> I<days>

Cache expiration time, in days (float point value). Default is 1/5 day.

=item	B<--acl-offline>, B<--no-acl-offline>.

B<--acl-offline> - assume offline, use old cache. Disabled by default.
Also can be enabled from environment variable ALTLINUX_ACL_OFFLINE=1
and explicitly disabled by B<--no-acl-offline>.

=item	B<--acl-verbose> I<verbosity level>

Internal debug/verbosity for ALTLinux::ACL.

=back


=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	COPYING

Copyright (c) 2010-2023 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

